/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package com.funambol.util;

/**
 * This exception represents the base exception for coded error conditions.
 * It makes possible for the receiver to analize the error condition using
 * one exception class.
 * 
 * This class defines only the basic codes. Classes deriving this this can
 * define other codes.
 */
public class CodedException extends RuntimeException {

    /** Storage error. (Problem accessing the backend storage, read or write) */
    public static final int STORAGE_ERROR     = 10;
    /** Out of memory error. It's not used at the moment */
    public static final int MEMORY_ERROR      = 11;
    /** The limit (memory, items) in the client has been reached */
    public static final int LIMIT_ERROR       = 12;

    /** Another sync is in progress */
    public static final int CONCURRENCE_ERROR = 100;

    /** The code of the exception */
    private int code;

    /**
     * Constructs an instance of <code>CodedException</code> with thei
     * code and specified detail message.
     * @param code the error code
     * @param msg the detail message.
     */
    public CodedException(int code, String msg) {
        super(msg);
        this.code = code;
    }

    /** Returns the code of this exception */
    public int getCode() {
        return code;
    }

}

